<?php

/**
 * 
 * Subs.ro API Parser - Implementation Practical Example
 * 
 * @author      Gabriel C. <gabi@subs.ro>
 * @copyright   2013 Gabriel C.
 * @link        http://api.subs.ro
 * @docs		http://api.subs.ro/docs/
 * @license     N/A
 * @version     2.0.0
 * @package     API Subs Parser
 * @depends     SimpleXMLElement, CURL
 * 
 * Connection reset by beer!
 * 
 */
class Subs_API_Parser {
    /**
     * API Base URL.
     */

    const API_BASE = 'http://api.subs.ro';

    /**
     * Accepted response types.
     * @var array
     */
    private $RESPONSE_TYPE = array('xml', 'json');

    /**
     * Accepted action types.
     * @var type 
     */
    private $ACTIONS = array('name', 'release', 'imdb');

    /**
     * Private properties.
     */
    private $type;
    private $action;
    private $searchParam;
    private $apiUrl;
    private $rawResponse;
    public $response;

    /**
     * 
     * Class' constructor.
     * 
     * @param string $type
     * @param string $action
     * @param string $search
     * @return object
     */
    public function Subs_API_Parser($type, $action, $search) {

        /*
         * Few checks.
         */
        if (!in_array($type, $this->RESPONSE_TYPE)) {
            die('Invalid response type chosen.');
        }
        if (!in_array($action, $this->ACTIONS)) {
            die('Invalid action type chosen.');
        }
        if (!$search) {
            die('No search parameter set.');
        }

        /*
         * Set these private properties.
         */
        $this->type = $type;
        $this->action = $action;
        $this->searchParam = $search;

        /*
         * Begin.
         */
        $this->process();

        /*
         * Return the response.
         */
        return $this->response;
    }

    /**
     * 
     * The entire parsing process will start up from here.
     */
    private function process() {

        $this->setApiUrl();
        $this->setResponse();
        $this->parseResponseByType();
    }

    /**
     * 
     * Set api url depending on sent options.
     */
    private function setApiUrl() {

        $this->apiUrl = join('/', array(
            self::API_BASE,
            $this->action,
            $this->type,
            urlencode($this->searchParam),
                ));
    }

    /**
     * 
     * Set the raw response appended from CURL request.
     */
    private function setResponse() {

        $this->rawResponse = $this->getResponseFromApi();
    }

    /**
     * 
     * Get the processed response.
     * @return array
     */
    public function getResponse() {

        return $this->response;
    }

    /**
     * 
     * @return array
     */
    private function getResponseFromApi() {

        /*
         * Set some CURL options.
         */
        $options = array(
            CURLOPT_RETURNTRANSFER => true, // return web page
            CURLOPT_HEADER => false, // don't return headers
            CURLOPT_FOLLOWLOCATION => true, // follow redirects
            CURLOPT_ENCODING => "", // handle compressed
            CURLOPT_AUTOREFERER => true, // set referer on redirect
            CURLOPT_CONNECTTIMEOUT => 120, // timeout on connect
            CURLOPT_TIMEOUT => 120, // timeout on response
            CURLOPT_MAXREDIRS => 10,
        );
        $ch = curl_init($this->apiUrl);
        curl_setopt_array($ch, $options);

        $content = curl_exec($ch);
        $err = curl_errno($ch);
        $errmsg = curl_error($ch);
        $header = curl_getinfo($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        curl_close($ch);

        /*
         * Format response. Other indexes other than 'content' are kept for
         * debugging purposes if needed.
         */
        $header['errno'] = $err;
        $header['errmsg'] = $errmsg;
        $header['content'] = $content;
        return $header;
    }

    /**
     * Parse the raw content depending on the response type.
     */
    private function parseResponseByType() {

        /*
         * Few checks.
         */
        if (!is_array($this->rawResponse)) {
            die('Invalid response received.');
        } elseif (empty($this->rawResponse['content'])) {
            die('Invalid response received.');
        }

        /*
         * Parse depending on the 
         */
        if ($this->type === 'xml') {
            $this->parseXml();
        } elseif ($this->type === 'json') {
            $this->parseJson();
        }
    }

    /**
     * 
     * Simple json_decode to set the response. 
     */
    private function parseJson() {

        $this->response = json_decode($this->rawResponse['content']);
    }

    /**
     * 
     * Parse the response as XML and set the response.
     */
    private function parseXml() {

        $xml = new SimpleXMLElement($this->rawResponse['content']);

        foreach ($xml->subtitle as $element) {
            foreach ($element as $key => $val) {
                $subtitle[$key] = (string) $val;
            }
            $this->response[] = (object) $subtitle;
            unset($subtitle);
        }
    }

}
